@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      basicio.cpp
-  Version:   $Rev: 2344 $
+  Version:   $Rev: 2453 $
   Author(s): Brad Schick (brad) <brad@robotbattle.com>
   History:   04-Dec-04, brad: created
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: basicio.cpp 2344 2010-09-10 14:02:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: basicio.cpp 2453 2011-02-13 14:08:44Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -382,7 +382,7 @@ namespace Exiv2 {
                 throw Error(2, path(), "MSG2", "DuplicateHandle");
             }
         }
-        p_->hMap_ = CreateFileMapping(p_->hFile_, 0, flProtect, 0, p_->mappedLength_, 0);
+        p_->hMap_ = CreateFileMapping(p_->hFile_, 0, flProtect, 0, (DWORD) p_->mappedLength_, 0);
         if (p_->hMap_ == 0 ) {
 #ifdef EXV_UNICODE_PATH
             if (p_->wpMode_ == Impl::wpUnicode) {
