@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      exif.cpp
-  Version:   $Rev: 2353 $
+  Version:   $Rev: 2453 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   26-Jan-04, ahu: created
              11-Feb-04, ahu: isolated as a component
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: exif.cpp 2353 2010-09-21 15:45:07Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: exif.cpp 2453 2011-02-13 14:08:44Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -793,10 +793,10 @@ namespace Exiv2 {
             }
         }
 
-        // Delete unknown tags larger than 4kB.
+        // Delete unknown tags larger than 4kB and known tags larger than 40kB.
         for (ExifData::iterator pos = ed.begin(); pos != ed.end(); ) {
-            if (   pos->size() > 4096
-                && pos->tagName().substr(0, 2) == "0x") {
+            if (   (pos->size() > 4096 && pos->tagName().substr(0, 2) == "0x")
+                || pos->size() > 40960) {
 #ifndef SUPPRESS_WARNINGS
                 EXV_WARNING << "Exif tag " << pos->key() << " not encoded\n";
 #endif
