@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -22,13 +22,13 @@
   Abstract:  Command line program to display and manipulate image metadata.
 
   File:      exiv2.cpp
-  Version:   $Rev: 2374 $
+  Version:   $Rev: 2597 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   10-Dec-03, ahu: created
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: exiv2.cpp 2374 2010-11-06 13:53:15Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: exiv2.cpp 2597 2011-09-02 15:28:52Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -176,7 +176,8 @@ int main(int argc, char* const argv[])
     params.cleanup();
     Exiv2::XmpParser::terminate();
 
-    return rc;
+    // Return a positive one byte code for better consistency across platforms
+    return static_cast<unsigned int>(rc) % 256;
 } // main
 
 // *****************************************************************************
@@ -205,8 +206,10 @@ void Params::cleanup()
 
 void Params::version(std::ostream& os) const
 {
-    os << EXV_PACKAGE_STRING << "\n"
-       << _("Copyright (C) 2004-2010 Andreas Huggel.\n")
+    bool  b64    = sizeof(void*)==8;
+    const char* sBuild = b64 ? "(64 bit build)" : "(32 bit build)" ;
+    os << EXV_PACKAGE_STRING << " " << Exiv2::versionNumberHexString() << " " << sBuild << "\n"
+       << _("Copyright (C) 2004-2011 Andreas Huggel.\n")
        << "\n"
        << _("This program is free software; you can redistribute it and/or\n"
             "modify it under the terms of the GNU General Public License\n"
@@ -904,7 +907,7 @@ namespace {
             case 'p':
             {
                 if (strcmp(action.c_str(), "extract") == 0) {
-                    i += parsePreviewNumbers(Params::instance().previewNumbers_, optarg, i + 1);
+                    i += (size_t) parsePreviewNumbers(Params::instance().previewNumbers_, optarg, (int) i + 1);
                     target |= Params::ctPreview;
                     break;
                 }
@@ -957,7 +960,7 @@ namespace {
         }
         std::cout << std::endl;
 #endif
-        return k - j;
+        return (int) (k - j);
     } // parsePreviewNumbers
 
     bool parseCmdFiles(ModifyCmds& modifyCmds,
