@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2009 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -21,7 +21,7 @@
 /*!
   @file    image.hpp
   @brief   Class Image, defining the interface for all Image subclasses.
-  @version $Rev: 1944 $
+  @version $Rev: 2553 $
   @author  Andreas Huggel (ahu)
            <a href="mailto:ahuggel@gmx.net">ahuggel@gmx.net</a>
   @author  Brad Schick (brad)
@@ -58,6 +58,19 @@ namespace Exiv2 {
         const int none = 0;         //!< Not an image
     }
 
+    //! Native preview information. This is meant to be used only by the PreviewManager.
+    struct NativePreview {
+        long position_;
+        uint32_t size_;
+        uint32_t width_;
+        uint32_t height_;
+        std::string filter_;
+        std::string mimeType_;
+    };
+
+    //! List of native previews. This is meant to be used only by the PreviewManager.
+    typedef std::vector<NativePreview> NativePreviewList;
+
     /*!
       @brief Abstract base class defining the interface for an image. This is
          the top-level interface to the Exiv2 library.
@@ -385,6 +398,8 @@ namespace Exiv2 {
         bool supportsMetadata(MetadataId metadataId) const;
         //! Return the flag indicating the source when writing XMP metadata.
         bool writeXmpFromPacket() const;
+        //! Return list of native previews. This is meant to be used only by the PreviewManager.
+        const NativePreviewList& nativePreviews() const;
         //@}
 
     protected:
@@ -397,6 +412,7 @@ namespace Exiv2 {
         std::string       xmpPacket_;         //!< XMP packet
         int               pixelWidth_;        //!< image pixel width
         int               pixelHeight_;       //!< image pixel height
+        NativePreviewList nativePreviews_;    //!< list of native previews
 
     private:
         //! @name NOT implemented
