@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,7 +20,7 @@
  */
 /*
   File:      jpgimage.cpp
-  Version:   $Rev: 2344 $
+  Version:   $Rev: 2558 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
              Brad Schick (brad) <brad@robotbattle.com>
              Volker Grabsch (vog) <vog@notjusthosting.com>
@@ -29,7 +29,7 @@
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: jpgimage.cpp 2344 2010-09-10 14:02:09Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: jpgimage.cpp 2558 2011-07-18 15:24:38Z vog $")
 
 // *****************************************************************************
 // included header files
@@ -88,6 +88,7 @@ namespace Exiv2 {
     const char     Photoshop::ps3Id_[] = "Photoshop 3.0\0";
     const char     Photoshop::bimId_[] = "8BIM";
     const uint16_t Photoshop::iptc_    = 0x0404;
+    const uint16_t Photoshop::preview_ = 0x040c;
 
     bool Photoshop::valid(const byte* pPsData,
                           long        sizePsData)
@@ -197,6 +198,16 @@ namespace Exiv2 {
                          record, sizeHdr, sizeData);
     }
 
+    int Photoshop::locatePreviewIrb(const byte*     pPsData,
+                                    long            sizePsData,
+                                    const byte**    record,
+                                    uint32_t *const sizeHdr,
+                                    uint32_t *const sizeData)
+    {
+        return locateIrb(pPsData, sizePsData, preview_,
+                         record, sizeHdr, sizeData);
+    }
+
     DataBuf Photoshop::setIptcIrb(const byte*     pPsData,
                                   long            sizePsData,
                                   const IptcData& iptcData)
@@ -392,7 +403,7 @@ namespace Exiv2 {
                 // Append to psBlob
                 append(psBlob, psData.pData_, psData.size_);
                 // Check whether psBlob is complete
-                if (psBlob.size() > 0 && Photoshop::valid(&psBlob[0], psBlob.size())) {
+                if (psBlob.size() > 0 && Photoshop::valid(&psBlob[0], (long) psBlob.size())) {
                     --search;
                     foundCompletePsData = true;
                 }
@@ -582,7 +593,7 @@ namespace Exiv2 {
                 // Append to psBlob
                 append(psBlob, psData.pData_, psData.size_);
                 // Check whether psBlob is complete
-                if (psBlob.size() > 0 && Photoshop::valid(&psBlob[0], psBlob.size())) {
+                if (psBlob.size() > 0 && Photoshop::valid(&psBlob[0],(long) psBlob.size())) {
                     foundCompletePsData = true;
                 }
             }
@@ -625,8 +636,8 @@ namespace Exiv2 {
             ++count;
         }
 
-        if (!foundCompletePsData && skipApp13Ps3.size() > 0) throw Error(22);
-        search += skipApp13Ps3.size();
+        if (!foundCompletePsData && psBlob.size() > 0) throw Error(22);
+        search += (int) skipApp13Ps3.size();
 
         if (comPos == 0) {
             if (marker == eoi_) comPos = count;
@@ -722,7 +733,7 @@ namespace Exiv2 {
                     // Set the new IPTC IRB, keeps existing IRBs but removes the
                     // IPTC block if there is no new IPTC data to write
                     DataBuf newPsData = Photoshop::setIptcIrb(psBlob.size() > 0 ? &psBlob[0] : 0,
-                                                              psBlob.size(),
+                                                              (long) psBlob.size(),
                                                               iptcData_);
                     const long maxChunkSize = 0xffff - 16;
                     const byte* chunkStart = newPsData.pData_;
