@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,14 +20,14 @@
  */
 /*
   File:      version.cpp
-  Version:   $Rev: 2286 $
+  Version:   $Rev: 2578 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   06-Mar-07, ahu: created
 
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: version.cpp 2286 2010-06-27 10:04:39Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: version.cpp 2578 2011-08-22 21:56:41Z vog $")
 
 // *****************************************************************************
 // included header files
@@ -39,12 +39,23 @@ EXIV2_RCSID("@(#) $Id: version.cpp 2286 2010-06-27 10:04:39Z ahuggel $")
 
 #include "version.hpp"
 
+// + standard includes
+#include <iomanip>
+#include <sstream>
+
 namespace Exiv2 {
     int versionNumber()
     {
         return EXIV2_MAKE_VERSION(EXIV2_MAJOR_VERSION, EXIV2_MINOR_VERSION, EXIV2_PATCH_VERSION);
     }
 
+    std::string versionNumberHexString()
+    {
+        std::ostringstream os;
+        os << std::hex << std::setw(6) << std::setfill('0') << Exiv2::versionNumber();
+        return os.str();
+    }
+
     const char* version()
     {
         return EXV_PACKAGE_VERSION;
