@@ -1,6 +1,6 @@
 // ***************************************************************** -*- C++ -*-
 /*
- * Copyright (C) 2004-2010 Andreas Huggel <ahuggel@gmx.net>
+ * Copyright (C) 2004-2011 Andreas Huggel <ahuggel@gmx.net>
  *
  * This program is part of the Exiv2 distribution.
  *
@@ -20,13 +20,13 @@
  */
 /*
   File:      xmp.cpp
-  Version:   $Rev: 2374 $
+  Version:   $Rev: 2538 $
   Author(s): Andreas Huggel (ahu) <ahuggel@gmx.net>
   History:   13-July-07, ahu: created
  */
 // *****************************************************************************
 #include "rcsid_int.hpp"
-EXIV2_RCSID("@(#) $Id: xmp.cpp 2374 2010-11-06 13:53:15Z ahuggel $")
+EXIV2_RCSID("@(#) $Id: xmp.cpp 2538 2011-07-01 15:03:45Z ahuggel $")
 
 // *****************************************************************************
 // included header files
@@ -101,6 +101,24 @@ namespace {
     //! Make an XMP key from a schema namespace and property path
     Exiv2::XmpKey::AutoPtr makeXmpKey(const std::string& schemaNs,
                                       const std::string& propPath);
+
+    //! Helper class used to serialize critical sections
+    class AutoLock
+    {
+    public:
+        AutoLock(Exiv2::XmpParser::XmpLockFct xmpLockFct, void* pLockData)
+            : xmpLockFct_(xmpLockFct), pLockData_(pLockData)
+        {
+            if (xmpLockFct_) xmpLockFct_(pLockData_, true);
+        }
+        ~AutoLock()
+        {
+            if (xmpLockFct_) xmpLockFct_(pLockData_, false);
+        }
+    private:
+        Exiv2::XmpParser::XmpLockFct xmpLockFct_;
+        void* pLockData_;
+    };
 }
 
 // *****************************************************************************
@@ -375,11 +393,15 @@ namespace Exiv2 {
     }
 
     bool XmpParser::initialized_ = false;
+    XmpParser::XmpLockFct XmpParser::xmpLockFct_ = 0;
+    void* XmpParser::pLockData_ = 0;
 
-    bool XmpParser::initialize()
+    bool XmpParser::initialize(XmpParser::XmpLockFct xmpLockFct, void* pLockData)
     {
         if (!initialized_) {
 #ifdef EXV_HAVE_XMP_TOOLKIT
+            xmpLockFct_ = xmpLockFct;
+            pLockData_ = pLockData;
             initialized_ = SXMPMeta::Initialize();
             SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/1.0/", "digiKam");
             SXMPMeta::RegisterNamespace("http://www.digikam.org/ns/kipi/1.0/", "kipi");
@@ -402,6 +424,8 @@ namespace Exiv2 {
 #ifdef EXV_HAVE_XMP_TOOLKIT
             SXMPMeta::Terminate();
 #endif
+            xmpLockFct_ = 0;
+            pLockData_ = 0;
             initialized_ = false;
         }
     }
@@ -412,6 +436,7 @@ namespace Exiv2 {
     {
         try {
             initialize();
+            AutoLock autoLock(xmpLockFct_, pLockData_);
             SXMPMeta::DeleteNamespace(ns.c_str());
             SXMPMeta::RegisterNamespace(ns.c_str(), prefix.c_str());
         }
@@ -420,11 +445,10 @@ namespace Exiv2 {
         }
     } // XmpParser::registerNs
 #else
-    bool XmpParser::registerNs(const std::string& /*ns*/,
+    void XmpParser::registerNs(const std::string& /*ns*/,
                                const std::string& /*prefix*/)
     {
         initialize();
-        return true;
     } // XmpParser::registerNs
 #endif
 
@@ -652,7 +676,7 @@ namespace Exiv2 {
             }
             // Todo: Xmpdatum should have an XmpValue, not a Value
             const XmpValue* val = dynamic_cast<const XmpValue*>(&i->value());
-            assert(val);
+            if (val == 0) throw Error(52, i->key(), i->typeName());
             options =   xmpArrayOptionBits(val->xmpArrayType())
                       | xmpArrayOptionBits(val->xmpStruct());
             if (   i->typeId() == xmpBag
@@ -687,7 +711,7 @@ namespace Exiv2 {
                 continue;
             }
             // Don't let any Xmpdatum go by unnoticed
-            throw Error(38, i->tagName(), TypeInfo::typeName(i->typeId()));
+            throw Error(38, i->tagName(), i->typeName());
         }
         std::string tmpPacket;
         meta.SerializeToBuffer(&tmpPacket, xmpFormatOptionBits(static_cast<XmpFormatFlags>(formatFlags)), padding); // throws
